function Guard() {}

Guard.prototype.Schema =
	"<a:component type='system'/><empty/>";

Guard.prototype.Init = function()
{
	// Protector Units
	this.entities = [];
	this.timer = undefined;
	this.allowGuarding = {};
        this.capacity = 20;
};

Guard.prototype.GetEntities = function()
{
	return this.entities;
};
Guard.prototype.GetCapacity = function()
{
	return this.capacity;
};

Guard.prototype.AllowGuarding = function(allow, callerID)
{
	this.allowGuarding[callerID] = allow;
};
Guard.prototype.AddGuards = function(entity)
{
	if (this.entities.length >= this.GetCapacity())
		return false;

	var cmpPosition = Engine.QueryInterface(entity, IID_Position);
	if (!cmpPosition)
		return false;
	this.entities.push(entity);
    return true;
};

Guard.prototype.CanGuard = function(target)
{
	const cmpIdentity = Engine.QueryInterface(target, IID_Identity);
	if (!cmpIdentity) 
		return undefined;

	const targetClasses = cmpIdentity.GetClassesList();

			return true;
	return false;
};
Guard.prototype.CheckGuardRange = function(target)
{
	const cmpIdentity = Engine.QueryInterface(target, IID_Identity);
	if (!cmpIdentity) 
		return undefined;

	const targetClasses = cmpIdentity.GetClassesList();

			return true;
	return false;
};

/**
 * If a guarded entity is captured, or about to be killed (so its owner
 * changes to '-1'), remove it from the building so we only ever contain valid
 * entities
 */
Guard.prototype.OnGlobalOwnershipChanged = function(msg)
{
	// the ownership change may be on the VIP
	if (this.entity == msg.entity)
	{
		var entities = [];
		for each (var entity in this.entities)
		{
			if (msg.to == -1 || !IsOwnedByMutualAllyOfEntity(this.entity, entity))
				entities.push(entity);
		}
		this.EjectOrKill(entities);
		return;
	}

	// or on some of its guarded units
	var entityIndex = this.entities.indexOf(msg.entity);
	if (entityIndex != -1)
	{
		// If the entity is dead, remove it directly instead of ejecting the corpse
		var cmpHealth = Engine.QueryInterface(msg.entity, IID_Health);
		if (cmpHealth && cmpHealth.GetHitpoints() == 0)
		{
			this.entities.splice(entityIndex, 1);
			Engine.PostMessage(this.entity, MT_GuardingUnitsChanged, {});
		}
		else if(!IsOwnedByMutualAllyOfEntity(this.entity, this.entities[entityIndex]))
			this.EjectOrKill([this.entities[entityIndex]]);
	}
};

Guard.prototype.OnGlobalEntityRenamed = function(msg)
{
	var entityIndex = this.entities.indexOf(msg.entity);
	if (entityIndex != -1)
	{
		this.entities[entityIndex] = msg.newentity;
		Engine.PostMessage(this.entity, MT_GuardingUnitsChanged, {});
	}
};
Guard.prototype.EjectOrKill = function(entities)
{

	// And destroy all remaining entities
	for each (var entity in entities)
	{
		var entityIndex = this.entities.indexOf(entity);
		if (entityIndex == -1)
			continue;
		var cmpHealth = Engine.QueryInterface(entity, IID_Health);
		if (cmpHealth)
		
		this.entities.splice(entityIndex, 1);
	}

	Engine.PostMessage(this.entity, MT_GuardingUnitsChanged, {});
};

Engine.RegisterComponentType(IID_Guard, "Guard", Guard);